import { select, csv, scaleLinear, scaleBand, axisLeft, axisBottom } from 'd3'

const svg = select("svg");

d3.csv("data.csv", function(data) {
    for (var i = 0; i < data.length; i++) {
        console.log(data[i].Name);
        console.log(data[i].Age);
    }
});
const process = (data) => {
  console.log(data);
  var groups = data.columns.slice(1)
  data.forEach(row => groups.forEach(key => row[key] = +row[key]))
  
  const yValue = d => d.date; 

  return {data, groups, yValue}
}

const render = (svg, {data, groups, yValue}) => {
  console.log('Rendering', data)
  const margin = {top: 20, right: 40, bottom: 20, left: 90} 
  const outer = {width: +svg.attr('width'), height: svg.attr('height')}
  const inner = {
    width: outer.width - margin.left - margin.right,
  	height:outer.height - margin.top - margin.bottom
  }
  
  var color = d3.scaleOrdinal()
    .domain(groups)
    .range(d3.schemeTableau10);
  
  let dataNegative = JSON.parse(JSON.stringify(data))
	let dataPositive = JSON.parse(JSON.stringify(data))
  
  dataNegative.forEach(entry => 
      groups.forEach(key => {entry[key] = entry[key] > 0 ? 0 : entry[key]})
  )

  dataPositive.forEach(entry => 
      groups.forEach(key => {entry[key] = entry[key] < 0 ? 0 : entry[key]})
  )
  
  var stackedDataPositive = d3.stack().keys(groups)(dataPositive)
  var stackedDataNegative = d3.stack().keys(groups)(dataNegative)
           
  const min_ = Math.min(...stackedDataNegative.map(v => 
                                  Math.min(...v.map(item=>item[1]))))
	const max_ = Math.max(...stackedDataPositive.map(v => 
                                  Math.max(...v.map(item=>item[1]))))
  
  const xDomain = [min_, max_];

  const xScale = scaleLinear()
    .domain(xDomain)
    .range([0, inner.width]);
  
  const yScale = scaleBand()
    .domain(data.map(yValue))
    .range([0, inner.height])
    .padding(0.2)
   
  const g = svg.append('g')
    .attr('transform', `translate(${margin.left}, ${margin.top})`)
  
	g.append('g').call(
     axisLeft(yScale)
      .tickPadding(5)
      .tickSizeInner(xScale(0))
  )
    .attr('transform', `translate(${xScale(0)}, 0)`)

	g.append('g').call(axisBottom(xScale))
    .attr('transform', `translate(0, ${inner.height})`)
  
  g.append('g').attr('class', 'bars')
    .selectAll('g.category')
    .data([
      {v: stackedDataPositive, t: 'positive'},
      {v: stackedDataNegative, t: 'negative'}
      ])
    .join('g').attr('class', d=>`category ${d.t}`)
    .selectAll('g')
    .data(d=>d.v)
    .join('g')
      .attr('x-group', (d)=>d.key)
			.attr("fill", (d)=>color(d.key))
      .selectAll('rect')
      .data(d=>d.filter(v=>v[0]!=v[1])) // no zero-sized bars
      .join('rect')
        .attr('height', yScale.bandwidth())
        .attr("x-date", (d)=>yValue(d.data))
        .attr("y", (d)=>yScale(yValue(d.data)))
			  .attr("x", (d)=> d[1] > 0 ? xScale(d[0]) : xScale(d[1]))
        .attr("width", (d)=>Math.abs(xScale(d[1]) - xScale(d[0])))   
};

csv('data.csv').then(process)
  .then((d)=>render(svg, d));